@@ -171,7 +171,9 @@ class TelegramBotManager(LocaleMixin):
             self.logger.debug("Webhook is set...")
 
         self.logger.debug("Checking connection to Telegram bot API...")
-        self.me: User = self.updater.bot.get_me()
+        me = self.updater.bot.get_me()
+        assert me, "Invalid bot credential provided."
+        self.me: User = me
         self.logger.debug("Connection to Telegram bot API is OK...")
         self.admins: List[int] = config['admins']
         self.dispatcher: Dispatcher = self.updater.dispatcher
@@ -477,6 +479,9 @@ class TelegramBotManager(LocaleMixin):
         return self.updater.bot.get_me(*args, **kwargs)
 
     def session_expired(self, update: Update, context: CallbackContext):
+        assert isinstance(update, Update)
+        assert update.effective_message
+        assert update.effective_chat
         if update.callback_query:
             update.callback_query.answer()
         self.edit_message_text(text=self._("Session expired. Please try again. (SE01)"),
