@@ -84,11 +84,17 @@ class CommandsManager(LocaleMixin):
         Returns:
             The next state
         """
+        assert isinstance(update, Update)
+        assert update.effective_chat
+        assert update.effective_message
+        assert update.callback_query
 
         chat_id = update.effective_chat.id
         message_id = update.effective_message.message_id
         callback = update.callback_query.data
 
+        assert callback
+
         index = (chat_id, message_id)
 
         if not callback.isdecimal():
@@ -104,10 +110,10 @@ class CommandsManager(LocaleMixin):
             update.callback_query.answer()
             return ConversationHandler.END
 
-        callback = int(callback)
+        callback_idx = int(callback)
         command_storage = self.msg_storage[index]
         module = command_storage.module
-        command = command_storage.commands[callback]
+        command = command_storage.commands[callback_idx]
         prefix = command_storage.prefix
 
         self.logger.debug("[%s.%s] Command execution callback is valid. Command storage item: %s", chat_id, message_id, command_storage)
@@ -148,6 +154,9 @@ class CommandsManager(LocaleMixin):
         Show list of additional features and their usage.
         Triggered by `/extra`.
         """
+        assert isinstance(update, Update)
+        assert update.effective_chat
+
         msg = self._("<i>Click the link next to the name for usage.</i>\n")
         for idx, i in enumerate(self.modules_list):
             if isinstance(i, Channel):
@@ -181,6 +190,10 @@ class CommandsManager(LocaleMixin):
         self.bot.send_message(update.effective_chat.id, msg, parse_mode="HTML")
 
     def extra_usage(self, update: Update, context: CallbackContext):
+        assert context.match
+        assert isinstance(update, Update)
+        assert update.effective_chat
+
         groupdict = context.match.groupdict()
         if int(groupdict['id']) >= len(self.modules_list):
             return self.bot.reply_error(update, self._("Invalid module ID. (XC03)"))
@@ -211,6 +224,10 @@ class CommandsManager(LocaleMixin):
         """
         Invoke an additional feature from slave channel.
         """
+        assert context.match
+        assert isinstance(update, Update)
+        assert update.message
+
         groupdict = context.match.groupdict()
         if int(groupdict['id']) >= len(coordinator.slaves):
             return self.bot.reply_error(update, self._("Invalid module ID. (XC01)"))
@@ -231,6 +248,7 @@ class CommandsManager(LocaleMixin):
         msg = self.bot.send_message(update.message.chat.id,
                                     prefix=header, text=self._("Please wait..."))
 
+        assert update.message.text
         result = functions[ExtraCommandName(groupdict['command'])](
             " ".join(update.message.text.split(' ', 1)[1:]))
 
