@@ -34,9 +34,11 @@ class LocaleHandler(Handler):
         self.channel = channel
         self.auto_locale = self.channel.flag('auto_locale')
 
-    def check_update(self, update: Update):
+    def check_update(self, update: object):
         if not self.auto_locale:
             return False
+        if not isinstance(update, Update):
+            return False
         if not update.effective_user or not update.effective_user.language_code:
             return
         self.logger.debug("[%s] Update has language %s.", update.update_id, update.effective_user.language_code)
